@@ -4,15 +4,14 @@
 import re
 
 replacements = [
-    (re.compile(r'pikepdf._qpdf.Object\b'), 'pikepdf.Object'),
-    (re.compile(r'pikepdf._qpdf.Pdf\b'), 'pikepdf.Pdf'),
-    (re.compile(r'pikepdf._qpdf.Page\b'), 'pikepdf.Page'),
+    (re.compile(r'pikepdf._qpdf.(\w+)\b'), r'pikepdf.\1'),
     (re.compile(r'QPDFTokenizer::Token\b'), 'pikepdf.Token'),
-    (re.compile(r'pikepdf._qpdf.Token\b'), 'pikepdf.Token'),
-    (re.compile(r'pikepdf._qpdf.TokenFilter\b'), 'pikepdf.TokenFilter'),
-    (re.compile(r'pikepdf._qpdf.TokenType\b'), 'pikepdf.TokenType'),
+    (re.compile(r'QPDFEFStreamObjectHelper'), 'pikepdf._qpdf.AttachedFile'),
+    (re.compile(r'QPDFObjectHandle::TokenFilter'), 'pikepdf.TokenFilter'),
+    (re.compile(r'QPDFObjectHandle::Rectangle'), 'pikepdf.Rectangle'),
     (re.compile(r'QPDFObjectHandle'), 'pikepdf.Object'),
     (re.compile(r'QPDFExc'), 'pikepdf.PdfError'),
+    (re.compile(r'QPDFPageObjectHelper'), 'pikepdf.Page'),
 ]
 
 
@@ -27,8 +26,10 @@ def fix_sigs(app, what, name, obj, options, signature, return_annotation):
 
 def fix_doc(app, what, name, obj, options, lines):
     for n, line in enumerate(lines[:]):
+        s = line
         for from_, to in replacements:
-            lines[n] = from_.sub(to, lines[n])
+            s = from_.sub(to, s)
+        lines[n] = s
 
 
 def setup(app):
