@@ -7,18 +7,39 @@
 import logging
 import re
 import sys
-from collections import namedtuple
-from collections.abc import MutableMapping
+from abc import ABC, abstractmethod
 from datetime import datetime
 from functools import wraps
 from io import BytesIO
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    Callable,
+    Dict,
+    List,
+    NamedTuple,
+    Optional,
+    Set,
+    Type,
+    Union,
+    cast,
+)
 from warnings import warn
 
 from lxml import etree
-from lxml.etree import QName, XMLParser, XMLSyntaxError, parse
+from lxml.etree import QName, XMLSyntaxError
 
-from .. import Name, PdfError, Stream, String
+from .. import Name, Stream, String
 from .. import __version__ as pikepdf_version
+from .._xml import parse_xml
+
+if sys.version_info < (3, 9):  # pragma: no cover
+    from typing import Iterable, MutableMapping
+else:
+    from collections.abc import Iterable, MutableMapping
+
+if TYPE_CHECKING:  # pragma: no cover
+    from pikepdf import Pdf
 
 XMP_NS_DC = "http://purl.org/dc/elements/1.1/"
 XMP_NS_PDF = "http://ns.adobe.com/pdf/1.3/"
@@ -63,13 +84,18 @@ XMP_EMPTY = b"""<x:xmpmeta xmlns:x="adobe:ns:meta/" x:xmptk="pikepdf">
 
 XPACKET_END = b"""\n<?xpacket end="w"?>\n"""
 
-XmpContainer = namedtuple('XmpContainer', ['rdf_type', 'py_type', 'insert_fn'])
+
+class XmpContainer(NamedTuple):
+    rdf_type: str
+    py_type: Type
+    insert_fn: Callable
+
 
 log = logging.getLogger(__name__)
 
 
 class NeverRaise(Exception):
-    """An exception that is never raised"""
+    """An exception that is never raised."""
 
     pass  # pylint: disable=unnecessary-pass
 
@@ -96,7 +122,7 @@ LANG_ALTS = frozenset(
 # These are the illegal characters in XML 1.0. (XML 1.1 is a bit more permissive,
 # but we'll be strict to ensure wider compatibility.)
 re_xml_illegal_chars = re.compile(
-    r"(?u)[^\x09\x0A\x0D\x20-\uD7FF\uE000-\uFFFD\u10000-\u10FFFF]"
+    r"(?u)[^\x09\x0A\x0D\x20-\U0000D7FF\U0000E000-\U0000FFFD\U00010000-\U0010FFFF]"
 )
 re_xml_illegal_bytes = re.compile(
     br"[^\x09\x0A\x0D\x20-\xFF]|&#0;"
@@ -104,6 +130,42 @@ re_xml_illegal_bytes = re.compile(
 )
 
 
+def _parser_basic(xml: bytes):
+    return parse_xml(BytesIO(xml))
+
+
+def _parser_strip_illegal_bytes(xml: bytes):
+    return parse_xml(BytesIO(re_xml_illegal_bytes.sub(b'', xml)))
+
+
+def _parser_recovery(xml: bytes):
+    return parse_xml(BytesIO(xml), recover=True)
+
+
+def _parser_replace_with_empty_xmp(_xml: bytes = b''):
+    log.warning("Error occurred parsing XMP, replacing with empty XMP.")
+    return _parser_basic(XMP_EMPTY)
+
+
+def _clean(s: Union[str, Iterable[str]], joiner: str = '; ') -> str:
+    """Ensure an object can safely be inserted in a XML tag body.
+
+    If we still have a non-str object at this point, the best option is to
+    join it, because it's apparently calling for a new node in a place that
+    isn't allowed in the spec or not supported.
+    """
+    if not isinstance(s, str):
+        if isinstance(s, Iterable):
+            warn(f"Merging elements of {s}")
+            if isinstance(s, Set):
+                s = joiner.join(sorted(s))
+            else:
+                s = joiner.join(s)
+        else:
+            raise TypeError("object must be a string or iterable of strings")
+    return re_xml_illegal_chars.sub('', s)
+
+
 def encode_pdf_date(d: datetime) -> str:
     """Encode Python datetime object as PDF date string
 
@@ -121,17 +183,19 @@ def encode_pdf_date(d: datetime) -> str:
     considered unknown. Regardless of whether or not GMT
     information is specified, the remainder of the string should specify
     the local time.
+
+    'D:' is required in PDF/A, so we always add it.
     """
 
     # The formatting of %Y is not consistent as described in
     # https://bugs.python.org/issue13305 and underspecification in libc.
     # So explicitly format the year with leading zeros
-    s = "{:04d}".format(d.year)
+    s = f"D:{d.year:04d}"
     s += d.strftime(r'%m%d%H%M%S')
     tz = d.strftime('%z')
     if tz:
         sign, tz_hours, tz_mins = tz[0], tz[1:3], tz[3:5]
-        s += "{}{}'{}'".format(sign, tz_hours, tz_mins)
+        s += f"{sign}{tz_hours}'{tz_mins}'"
     return s
 
 
@@ -159,36 +223,34 @@ def decode_pdf_date(s: str) -> datetime:
         return datetime.strptime(s, r'%Y%m%d%H%M%S')
 
 
-class AuthorConverter:
+class Converter(ABC):
     @staticmethod
-    def xmp_from_docinfo(docinfo_val):
+    @abstractmethod
+    def xmp_from_docinfo(docinfo_val: Optional[str]) -> Any:  # type: ignore
+        "Derive XMP metadata from a DocumentInfo string"
+
+    @staticmethod
+    @abstractmethod
+    def docinfo_from_xmp(xmp_val: Any) -> Optional[str]:
+        "Derive a DocumentInfo value from equivalent XMP metadata"
+
+
+class AuthorConverter(Converter):
+    @staticmethod
+    def xmp_from_docinfo(docinfo_val: Optional[str]) -> Any:  # type: ignore
         return [docinfo_val]
 
     @staticmethod
     def docinfo_from_xmp(xmp_val):
         if isinstance(xmp_val, str):
             return xmp_val
+        elif xmp_val is None or xmp_val == [None]:
+            return None
         else:
             return '; '.join(xmp_val)
 
 
-if sys.version_info < (3, 7):
-
-    def fromisoformat(datestr):
-        # strptime %z can't parse a timezone with punctuation
-        if re.search(r'[+-]\d{2}[-:]\d{2}$', datestr):
-            datestr = datestr[:-3] + datestr[-2:]
-        try:
-            return datetime.strptime(datestr, "%Y-%m-%dT%H:%M:%S%z")
-        except ValueError:
-            return datetime.strptime(datestr, "%Y-%m-%dT%H:%M:%S")
-
-
-else:
-    fromisoformat = datetime.fromisoformat
-
-
-class DateConverter:
+class DateConverter(Converter):
     @staticmethod
     def xmp_from_docinfo(docinfo_val):
         if docinfo_val == '':
@@ -199,10 +261,21 @@ class DateConverter:
     def docinfo_from_xmp(xmp_val):
         if xmp_val.endswith('Z'):
             xmp_val = xmp_val[:-1] + '+00:00'
-        dateobj = fromisoformat(xmp_val)
+        try:
+            dateobj = datetime.fromisoformat(xmp_val)
+        except IndexError:
+            # PyPy 3.7 may raise IndexError - convert to ValueError
+            raise ValueError(f"Invalid isoformat string: '{xmp_val}'") from None
         return encode_pdf_date(dateobj)
 
 
+class DocinfoMapping(NamedTuple):
+    ns: str
+    key: str
+    name: Name
+    converter: Optional[Type[Converter]]
+
+
 def ensure_loaded(fn):
     @wraps(fn)
     def wrapper(self, *args, **kwargs):
@@ -244,22 +317,34 @@ class PdfMetadata(MutableMapping):
         :meth:`pikepdf.Pdf.open_metadata`
     """
 
-    DOCINFO_MAPPING = [
-        (XMP_NS_DC, 'creator', Name.Author, AuthorConverter),
-        (XMP_NS_DC, 'description', Name.Subject, None),
-        (XMP_NS_DC, 'title', Name.Title, None),
-        (XMP_NS_PDF, 'Keywords', Name.Keywords, None),
-        (XMP_NS_PDF, 'Producer', Name.Producer, None),
-        (XMP_NS_XMP, 'CreateDate', Name.CreationDate, DateConverter),
-        (XMP_NS_XMP, 'CreatorTool', Name.Creator, None),
-        (XMP_NS_XMP, 'ModifyDate', Name.ModDate, DateConverter),
+    DOCINFO_MAPPING: List[DocinfoMapping] = [
+        DocinfoMapping(XMP_NS_DC, 'creator', Name.Author, AuthorConverter),
+        DocinfoMapping(XMP_NS_DC, 'description', Name.Subject, None),
+        DocinfoMapping(XMP_NS_DC, 'title', Name.Title, None),
+        DocinfoMapping(XMP_NS_PDF, 'Keywords', Name.Keywords, None),
+        DocinfoMapping(XMP_NS_PDF, 'Producer', Name.Producer, None),
+        DocinfoMapping(XMP_NS_XMP, 'CreateDate', Name.CreationDate, DateConverter),
+        DocinfoMapping(XMP_NS_XMP, 'CreatorTool', Name.Creator, None),
+        DocinfoMapping(XMP_NS_XMP, 'ModifyDate', Name.ModDate, DateConverter),
     ]
 
-    NS = {prefix: uri for uri, prefix in DEFAULT_NAMESPACES}
-    REVERSE_NS = {uri: prefix for uri, prefix in DEFAULT_NAMESPACES}
+    NS: Dict[str, str] = {prefix: uri for uri, prefix in DEFAULT_NAMESPACES}
+    REVERSE_NS: Dict[str, str] = {uri: prefix for uri, prefix in DEFAULT_NAMESPACES}
+
+    _PARSERS_OVERWRITE_INVALID_XML: Iterable[Callable[[bytes], Any]] = [
+        _parser_basic,
+        _parser_strip_illegal_bytes,
+        _parser_recovery,
+        _parser_replace_with_empty_xmp,
+    ]
+    _PARSERS_STANDARD: Iterable[Callable[[bytes], Any]] = [_parser_basic]
 
     def __init__(
-        self, pdf, pikepdf_mark=True, sync_docinfo=True, overwrite_invalid_xml=True
+        self,
+        pdf: 'Pdf',
+        pikepdf_mark: bool = True,
+        sync_docinfo: bool = True,
+        overwrite_invalid_xml: bool = True,
     ):
         self._pdf = pdf
         self._xmp = None
@@ -268,7 +353,9 @@ class PdfMetadata(MutableMapping):
         self._updating = False
         self.overwrite_invalid_xml = overwrite_invalid_xml
 
-    def load_from_docinfo(self, docinfo, delete_missing=False, raise_failure=False):
+    def load_from_docinfo(
+        self, docinfo, delete_missing: bool = False, raise_failure: bool = False
+    ) -> None:
         """Populate the XMP metadata object with DocumentInfo
 
         Arguments:
@@ -282,6 +369,13 @@ class PdfMetadata(MutableMapping):
         approximately equivalent to certain XMP records. This method copies
         those entries into the XMP metadata.
         """
+
+        def warn_or_raise(msg, e=None):
+            if raise_failure:
+                raise ValueError(msg) from e
+            else:
+                warn(msg)
+
         for uri, shortkey, docinfo_name, converter in self.DOCINFO_MAPPING:
             qname = QName(uri, shortkey)
             # docinfo might be a dict or pikepdf.Dictionary, so lookup keys
@@ -297,84 +391,66 @@ class PdfMetadata(MutableMapping):
                     val = converter.xmp_from_docinfo(val)
                 if not val:
                     continue
-                self[qname] = val
-            except (ValueError, AttributeError) as e:
-                msg = "The metadata field {} could not be copied to XMP".format(
-                    docinfo_name
+                self._setitem(qname, val, True)
+            except (ValueError, AttributeError, NotImplementedError) as e:
+                warn_or_raise(
+                    f"The metadata field {docinfo_name} could not be copied to XMP", e
                 )
-                if raise_failure:
-                    raise ValueError(msg) from e
-                else:
-                    warn(msg)
-        valid_docinfo_names = set(
+        valid_docinfo_names = {
             str(docinfo_name) for _, _, docinfo_name, _ in self.DOCINFO_MAPPING
-        )
-        extra_docinfo_names = set(str(k) for k in docinfo.keys()) - valid_docinfo_names
+        }
+        extra_docinfo_names = {str(k) for k in docinfo.keys()} - valid_docinfo_names
         for extra in extra_docinfo_names:
-            msg = (
-                "The metadata field {} with value '{}' has no XMP equivalent, "
-                "so it was discarded"
-            ).format(extra, docinfo.get(extra))
-            if raise_failure:
-                raise ValueError(msg)
-            else:
-                warn(msg)
+            warn_or_raise(
+                f"The metadata field {extra} with value '{repr(docinfo.get(extra))}' "
+                "has no XMP equivalent, so it was discarded",
+            )
 
-    def _load(self):
+    def _load(self) -> None:
         try:
             data = self._pdf.Root.Metadata.read_bytes()
         except AttributeError:
-            data = XMP_EMPTY
+            data = b''
         self._load_from(data)
 
-    def _load_from(self, data):
+    def _load_from(self, data: bytes) -> None:
         if data.strip() == b'':
             data = XMP_EMPTY  # on some platforms lxml chokes on empty documents
 
-        def basic_parser(xml):
-            return parse(BytesIO(xml))
-
-        def strip_illegal_bytes_parser(xml):
-            return parse(BytesIO(re_xml_illegal_bytes.sub(b'', xml)))
-
-        def recovery_parser(xml):
-            parser = XMLParser(recover=True)
-            return parse(BytesIO(xml), parser)
-
-        def replace_with_empty_xmp(xml=None):
-            log.warning("Error occurred parsing XMP, replacing with empty XMP.")
-            return basic_parser(XMP_EMPTY)
-
-        if self.overwrite_invalid_xml:
-            parsers = [
-                basic_parser,
-                strip_illegal_bytes_parser,
-                recovery_parser,
-                replace_with_empty_xmp,
-            ]
-        else:
-            parsers = [basic_parser]
+        parsers = (
+            self._PARSERS_OVERWRITE_INVALID_XML
+            if self.overwrite_invalid_xml
+            else self._PARSERS_STANDARD
+        )
 
         for parser in parsers:
             try:
                 self._xmp = parser(data)
-            except (XMLSyntaxError if self.overwrite_invalid_xml else NeverRaise) as e:
+            except (
+                XMLSyntaxError
+                if self.overwrite_invalid_xml
+                else NeverRaise  # type: ignore
+            ) as e:
                 if str(e).startswith("Start tag expected, '<' not found") or str(
                     e
                 ).startswith("Document is empty"):
-                    self._xmp = replace_with_empty_xmp()
+                    self._xmp = _parser_replace_with_empty_xmp()
                     break
             else:
                 break
 
-        try:
-            pis = self._xmp.xpath('/processing-instruction()')
-            for pi in pis:
-                etree.strip_tags(self._xmp, pi.tag)
-            self._get_rdf_root()
-        except (Exception if self.overwrite_invalid_xml else NeverRaise) as e:
-            log.warning("Error occurred parsing XMP", exc_info=e)
-            self._xmp = replace_with_empty_xmp()
+        if self._xmp is not None:
+            try:
+                pis = self._xmp.xpath('/processing-instruction()')
+                for pi in pis:
+                    etree.strip_tags(self._xmp, pi.tag)
+                self._get_rdf_root()
+            except (Exception if self.overwrite_invalid_xml else NeverRaise) as e:
+                log.warning("Error occurred parsing XMP", exc_info=e)
+                self._xmp = _parser_replace_with_empty_xmp()
+        else:
+            log.warning("Error occurred parsing XMP")
+            self._xmp = _parser_replace_with_empty_xmp()
         return
 
     @ensure_loaded
@@ -396,7 +472,8 @@ class PdfMetadata(MutableMapping):
         The standard mapping is described here:
             https://www.pdfa.org/pdfa-metadata-xmp-rdf-dublin-core/
         """
-        self._pdf.docinfo  # Touch object to ensure it exists
+        # Touch object to ensure it exists
+        self._pdf.docinfo  # pylint: disable=pointless-statement
         for uri, element, docinfo_name, converter in self.DOCINFO_MAPPING:
             qname = QName(uri, element)
             try:
@@ -410,16 +487,20 @@ class PdfMetadata(MutableMapping):
                     value = converter.docinfo_from_xmp(value)
                 except ValueError:
                     warn(
-                        "The DocumentInfo field {} could not be updated from XMP".format(
-                            docinfo_name
-                        )
+                        f"The DocumentInfo field {docinfo_name} could not be "
+                        "updated from XMP"
                     )
                     value = None
+                except Exception as e:
+                    raise ValueError(
+                        "An error occurred while updating DocumentInfo field "
+                        f"{docinfo_name} from XMP {qname} with value {value}"
+                    ) from e
             if value is None:
                 if docinfo_name in self._pdf.docinfo:
                     del self._pdf.docinfo[docinfo_name]
                 continue
-            value = re_xml_illegal_chars.sub('', value)
+            value = _clean(value)
             try:
                 # Try to save pure ASCII
                 self._pdf.docinfo[docinfo_name] = value.encode('ascii')
@@ -444,8 +525,17 @@ class PdfMetadata(MutableMapping):
         Depending how we are initialized, leave our metadata mark and producer.
         """
         if self.mark:
-            self[QName(XMP_NS_XMP, 'MetadataDate')] = datetime.now().isoformat()
-            self[QName(XMP_NS_PDF, 'Producer')] = 'pikepdf ' + pikepdf_version
+            # We were asked to mark the file as being edited by pikepdf
+            self._setitem(
+                QName(XMP_NS_XMP, 'MetadataDate'),
+                datetime.now(datetime.utcnow().astimezone().tzinfo).isoformat(),
+                applying_mark=True,
+            )
+            self._setitem(
+                QName(XMP_NS_PDF, 'Producer'),
+                'pikepdf ' + pikepdf_version,
+                applying_mark=True,
+            )
         xml = self._get_xml_bytes()
         self._pdf.Root.Metadata = Stream(self._pdf, xml)
         self._pdf.Root.Metadata[Name.Type] = Name.Metadata
@@ -453,22 +543,23 @@ class PdfMetadata(MutableMapping):
         if self.sync_docinfo:
             self._update_docinfo()
 
-    def _qname(self, name):
+    @classmethod
+    def _qname(cls, name: Union[QName, str]) -> str:
         """Convert name to an XML QName
 
         e.g. pdf:Producer -> {http://ns.adobe.com/pdf/1.3/}Producer
         """
         if isinstance(name, QName):
-            return name
+            return str(name)
         if not isinstance(name, str):
-            raise TypeError("{} must be str".format(name))
+            raise TypeError(f"{name} must be str")
         if name == '':
             return name
         if name.startswith('{'):
             return name
         prefix, tag = name.split(':', maxsplit=1)
-        uri = self.NS[prefix]
-        return QName(uri, tag)
+        uri = cls.NS[prefix]
+        return str(QName(uri, tag))
 
     def _prefix_from_uri(self, uriname):
         """Given a fully qualified XML name, find a prefix
@@ -494,7 +585,7 @@ class PdfMetadata(MutableMapping):
                 return ''
 
         for xmlcontainer, container, insertfn in XMP_CONTAINERS:
-            items = node.find('rdf:{}'.format(xmlcontainer), self.NS)
+            items = node.find(f'rdf:{xmlcontainer}', self.NS)
             if items is None:
                 continue
             result = container()
@@ -511,7 +602,7 @@ class PdfMetadata(MutableMapping):
                 raise ValueError("Metadata seems to be XML but not XMP")
         return rdf
 
-    def _get_elements(self, name=''):
+    def _get_elements(self, name: Union[str, QName] = ''):
         """Get elements from XMP
 
         Core routine to find elements matching name within the XMP and yield
@@ -527,7 +618,7 @@ class PdfMetadata(MutableMapping):
         the elements. We ignore qualifiers.
 
         Args:
-            name (str): a prefixed name or QName to look for within the
+            name: a prefixed name or QName to look for within the
                 data section of the XMP; looks for all data keys if omitted
 
         Yields:
@@ -555,18 +646,15 @@ class PdfMetadata(MutableMapping):
         yield from (v[2] for v in self._get_elements(name))
 
     @ensure_loaded
-    def __contains__(self, key):
-        try:
-            return any(self._get_element_values(key))
-        except KeyError:
-            return False
+    def __contains__(self, key: Union[str, QName]):
+        return any(self._get_element_values(key))
 
     @ensure_loaded
-    def __getitem__(self, key):
+    def __getitem__(self, key: Union[str, QName]):
         try:
             return next(self._get_element_values(key))
         except StopIteration:
-            raise KeyError(key)
+            raise KeyError(key) from None
 
     @ensure_loaded
     def __iter__(self):
@@ -580,74 +668,115 @@ class PdfMetadata(MutableMapping):
     def __len__(self):
         return len(list(iter(self)))
 
-    @ensure_loaded
-    def __setitem__(self, key, val):
+    def _setitem(
+        self,
+        key: Union[str, QName],
+        val: Union[Set[str], List[str], str],
+        applying_mark: bool = False,
+    ):
         if not self._updating:
             raise RuntimeError("Metadata not opened for editing, use with block")
 
-        def clean(s):
-            return re_xml_illegal_chars.sub('', s)
-
-        def add_array(node, items):
-            rdf_type = next(
-                c.rdf_type for c in XMP_CONTAINERS if isinstance(items, c.py_type)
-            )
-            seq = etree.SubElement(node, QName(XMP_NS_RDF, rdf_type))
-            if rdf_type == 'Alt':
-                attrib = {QName(XMP_NS_XML, 'lang'): 'x-default'}
-            else:
-                attrib = None
-            for item in items:
-                el = etree.SubElement(seq, QName(XMP_NS_RDF, 'li'), attrib=attrib)
-                el.text = clean(item)
+        qkey = self._qname(key)
+        self._setitem_check_args(key, val, applying_mark, qkey)
 
         try:
-            # Locate existing node to replace
-            node, attrib, _oldval, _parent = next(self._get_elements(key))
-            if attrib:
-                if not isinstance(val, str):
-                    raise TypeError(val)
-                node.set(attrib, clean(val))
-            elif isinstance(val, (list, set)):
-                for child in node.findall('*'):
-                    node.remove(child)
-                add_array(node, val)
-            elif isinstance(val, str):
-                for child in node.findall('*'):
-                    node.remove(child)
-                if str(self._qname(key)) in LANG_ALTS:
-                    add_array(node, AltList([clean(val)]))
-                else:
-                    node.text = clean(val)
-            else:
-                raise TypeError(val)
+            # Update existing node
+            self._setitem_update(key, val, qkey)
         except StopIteration:
             # Insert a new node
-            rdf = self._get_rdf_root()
-            if str(self._qname(key)) in LANG_ALTS:
-                val = AltList([clean(val)])
-            if isinstance(val, (list, set)):
-                rdfdesc = etree.SubElement(
-                    rdf,
-                    QName(XMP_NS_RDF, 'Description'),
-                    attrib={QName(XMP_NS_RDF, 'about'): ''},
-                )
-                node = etree.SubElement(rdfdesc, self._qname(key))
-                add_array(node, val)
-            elif isinstance(val, str):
-                rdfdesc = etree.SubElement(
-                    rdf,
-                    QName(XMP_NS_RDF, 'Description'),
-                    attrib={
-                        QName(XMP_NS_RDF, 'about'): '',
-                        self._qname(key): clean(val),
-                    },
-                )
+            self._setitem_insert(key, val)
+
+    def _setitem_check_args(self, key, val, applying_mark: bool, qkey: str) -> None:
+        if (
+            self.mark
+            and not applying_mark
+            and qkey
+            in (
+                self._qname('xmp:MetadataDate'),
+                self._qname('pdf:Producer'),
+            )
+        ):
+            # Complain if user writes self[pdf:Producer] = ... and because it will
+            # be overwritten on save, unless self._updating_mark, in which case
+            # the action was initiated internally
+            log.warning(
+                f"Update to {key} will be overwritten because metadata was opened "
+                "with set_pikepdf_as_editor=True"
+            )
+        if isinstance(val, str) and qkey in (self._qname('dc:creator')):
+            log.error(f"{key} should be set to a list of strings")
+
+    def _setitem_add_array(self, node, items: Iterable) -> None:
+        rdf_type = next(
+            c.rdf_type for c in XMP_CONTAINERS if isinstance(items, c.py_type)
+        )
+        seq = etree.SubElement(node, str(QName(XMP_NS_RDF, rdf_type)))
+        tag_attrib: Optional[Dict[str, str]] = None
+        if rdf_type == 'Alt':
+            tag_attrib = {str(QName(XMP_NS_XML, 'lang')): 'x-default'}
+        for item in items:
+            el = etree.SubElement(seq, str(QName(XMP_NS_RDF, 'li')), attrib=tag_attrib)
+            el.text = _clean(item)
+
+    def _setitem_update(self, key, val, qkey):
+        # Locate existing node to replace
+        node, attrib, _oldval, _parent = next(self._get_elements(key))
+        if attrib:
+            if not isinstance(val, str):
+                if qkey == self._qname('dc:creator'):
+                    # dc:creator incorrectly created as an attribute - we're
+                    # replacing it anyway, so remove the old one
+                    del node.attrib[qkey]
+                    self._setitem_add_array(node, _clean(val))
+                else:
+                    raise TypeError(f"Setting {key} to {val} with type {type(val)}")
             else:
-                raise TypeError(val)
+                node.set(attrib, _clean(val))
+        elif isinstance(val, (list, set)):
+            for child in node.findall('*'):
+                node.remove(child)
+            self._setitem_add_array(node, val)
+        elif isinstance(val, str):
+            for child in node.findall('*'):
+                node.remove(child)
+            if str(self._qname(key)) in LANG_ALTS:
+                self._setitem_add_array(node, AltList([_clean(val)]))
+            else:
+                node.text = _clean(val)
+        else:
+            raise TypeError(f"Setting {key} to {val} with type {type(val)}")
+
+    def _setitem_insert(self, key, val):
+        rdf = self._get_rdf_root()
+        if str(self._qname(key)) in LANG_ALTS:
+            val = AltList([_clean(val)])
+        if isinstance(val, (list, set)):
+            rdfdesc = etree.SubElement(
+                rdf,
+                str(QName(XMP_NS_RDF, 'Description')),
+                attrib={str(QName(XMP_NS_RDF, 'about')): ''},
+            )
+            node = etree.SubElement(rdfdesc, self._qname(key))
+            self._setitem_add_array(node, val)
+        elif isinstance(val, str):
+            _rdfdesc = etree.SubElement(  # lgtm [py/unused-local-variable]
+                rdf,
+                str(QName(XMP_NS_RDF, 'Description')),
+                attrib={
+                    QName(XMP_NS_RDF, 'about'): '',
+                    self._qname(key): _clean(val),
+                },
+            )
+        else:
+            raise TypeError(f"Setting {key} to {val} with type {type(val)}") from None
 
     @ensure_loaded
-    def __delitem__(self, key):
+    def __setitem__(self, key: Union[str, QName], val: Union[Set[str], List[str], str]):
+        return self._setitem(key, val, False)
+
+    @ensure_loaded
+    def __delitem__(self, key: Union[str, QName]):
         if not self._updating:
             raise RuntimeError("Metadata not opened for editing, use with block")
         try:
@@ -664,11 +793,10 @@ class PdfMetadata(MutableMapping):
             else:
                 parent.remove(node)
         except StopIteration:
-            raise KeyError(key)
+            raise KeyError(key) from None
 
     @property
-    @ensure_loaded
-    def pdfa_status(self):
+    def pdfa_status(self) -> str:
         """Returns the PDF/A conformance level claimed by this PDF, or False
 
         A PDF may claim to PDF/A compliant without this being true. Use an
@@ -676,10 +804,14 @@ class PdfMetadata(MutableMapping):
         conformant.
 
         Returns:
-            str: The conformance level of the PDF/A, or an empty string if the
+            The conformance level of the PDF/A, or an empty string if the
             PDF does not claim PDF/A conformance. Possible valid values
             are: 1A, 1B, 2A, 2B, 2U, 3A, 3B, 3U.
         """
+        # do same as @ensure_loaded - mypy can't handle decorated property
+        if not self._xmp:
+            self._load()
+
         key_part = QName(XMP_NS_PDFA_ID, 'part')
         key_conformance = QName(XMP_NS_PDFA_ID, 'conformance')
         try:
@@ -688,8 +820,7 @@ class PdfMetadata(MutableMapping):
             return ''
 
     @property
-    @ensure_loaded
-    def pdfx_status(self):
+    def pdfx_status(self) -> str:
         """Returns the PDF/X conformance level claimed by this PDF, or False
 
         A PDF may claim to PDF/X compliant without this being true. Use an
@@ -697,9 +828,13 @@ class PdfMetadata(MutableMapping):
         conformant.
 
         Returns:
-            str: The conformance level of the PDF/X, or an empty string if the
+            The conformance level of the PDF/X, or an empty string if the
             PDF does not claim PDF/X conformance.
         """
+        # do same as @ensure_loaded - mypy can't handle decorated property
+        if not self._xmp:
+            self._load()
+
         pdfx_version = QName(XMP_NS_PDFX_ID, 'GTS_PDFXVersion')
         try:
             return self[pdfx_version]
