@@ -14,11 +14,11 @@
 /* POSIX functions on Windows have a leading underscore
  */
 #if defined(_WIN32)
-#   define posix_fdopen _fdopen
-#   define posix_close  _close
+#    define posix_fdopen _fdopen
+#    define posix_close _close
 #else
-#   define posix_fdopen fdopen
-#   define posix_close  close
+#    define posix_fdopen fdopen
+#    define posix_close close
 #endif
 
 /* Convert a Python object to a filesystem encoded path
@@ -27,22 +27,10 @@
  * Cast to a string without transcoding.
  */
 
-#if PY_VERSION_HEX < 0x03060000
-
-    py::object fspath(py::object filename)
-    {
-        auto py_fspath = py::module::import("pikepdf._cpphelpers").attr("fspath");
-        return py_fspath(filename);
-    }
-
-#else
-
-    py::object fspath(py::object filename)
-    {
-        py::handle handle = PyOS_FSPath(filename.ptr());
-        if (!handle)
-            throw py::error_already_set();
-        return py::reinterpret_steal<py::object>(handle);
-    }
-
-#endif
+py::object fspath(py::object filename)
+{
+    py::handle handle = PyOS_FSPath(filename.ptr());
+    if (!handle)
+        throw py::error_already_set();
+    return py::reinterpret_steal<py::object>(handle);
+}
