@@ -17,3 +17,13 @@ inline bool str_startswith(T haystack, S needle)
 {
     return std::string(haystack).rfind(needle, 0) == 0;
 }
+
+template <typename T>
+inline bool str_replace(std::string &str, T from, T to)
+{
+    size_t start_pos = str.find(from);
+    if (start_pos == std::string::npos)
+        return false;
+    str.replace(start_pos, std::string(from).length(), to);
+    return true;
+}
\ No newline at end of file
