@@ -26,10 +26,20 @@ def graph_encrypted(resources):
 )
 def test_encrypt_basic(trivial, outpdf, R, owner, user):
     trivial.save(outpdf, encryption=dict(R=R, owner=owner, user=user))
-    pdf_owner = pikepdf.open(outpdf, password=owner)
-    assert pdf_owner.is_encrypted
-    pdf_user = pikepdf.open(outpdf, password=user)
-    assert pdf_user.is_encrypted
+    with pikepdf.open(outpdf, password=owner) as pdf_owner:
+        assert pdf_owner.is_encrypted
+        assert pdf_owner.owner_password_matched
+        if owner != user:
+            assert not pdf_owner.user_password_matched
+        else:
+            assert pdf_owner.user_password_matched
+    with pikepdf.open(outpdf, password=user) as pdf_user:
+        assert pdf_user.is_encrypted
+        assert pdf_user.user_password_matched
+        if owner != user:
+            assert not pdf_user.owner_password_matched
+        else:
+            assert pdf_user.owner_password_matched
 
 
 def test_encrypt_R5(trivial, outpdf):
@@ -57,21 +67,33 @@ def test_encrypt_no_passwords(trivial, outpdf):
     trivial.save(outpdf, encryption=dict(R=6))
 
 
-def test_encrypt_permissions_deny(trivial, outpdf):
-    perms = pikepdf.models.Permissions(extract=False)
+@pytest.mark.parametrize(
+    'highres, lowres', [(True, True), (False, True), (False, False)]
+)
+def test_encrypt_permissions_deny(trivial, outpdf, highres, lowres):
+    perms = pikepdf.models.Permissions(
+        extract=False, print_highres=highres, print_lowres=lowres
+    )
     trivial.save(
         outpdf, encryption=pikepdf.Encryption(owner='sun', user='moon', allow=perms)
     )
-    pdf = pikepdf.open(outpdf, password='sun')
-    assert not pdf.allow.extract
-    assert pdf.allow.modify_form
+    with pikepdf.open(outpdf, password='sun') as pdf:
+        assert not pdf.allow.extract
+        assert pdf.allow.modify_form
 
 
 def test_encrypt_info(trivial, outpdf):
     trivial.save(outpdf, encryption=dict(R=4, owner='foo', user='bar'))
-    pdf = pikepdf.open(outpdf, password='foo')
-    assert pdf.encryption.user_password == b'bar'
-    assert pdf.encryption.bits == 128
+    with pikepdf.open(outpdf, password='foo') as pdf:
+        assert pdf.encryption.user_password == b'bar'
+        assert pdf.encryption.bits == 128
+        assert pdf.encryption.R == 4
+        assert pdf.encryption.V == 4
+        assert pdf.encryption.P == -3392
+        assert pdf.encryption.stream_method == pikepdf._qpdf.EncryptionMethod.aes
+        assert pdf.encryption.string_method == pikepdf._qpdf.EncryptionMethod.aes
+        assert pdf.encryption.file_method == pikepdf._qpdf.EncryptionMethod.aes
+        assert pdf.encryption.encryption_key[:2] == b'\x02\xdc'
 
 
 @pytest.mark.parametrize(
@@ -123,7 +145,7 @@ def test_save_without_encryption(graph_encrypted, outpdf):
 
 def test_save_preserve_encryption(graph_encrypted, outpdf):
     graph_encrypted.save(outpdf, encryption=True)
-    with pikepdf.open(outpdf, 'owner') as pdf:
+    with pikepdf.open(outpdf, password='owner') as pdf:
         assert pdf.is_encrypted
 
 
