@@ -15,11 +15,12 @@ from pikepdf import Pdf, PdfError
 @pytest.fixture
 def sandwich(resources):
     # Has XMP, docinfo, <?adobe-xap-filters esc="CRLF"?>, shorthand attribute XMP
-    return Pdf.open(resources / 'sandwich.pdf')
+    with Pdf.open(resources / 'sandwich.pdf') as pdf:
+        yield pdf
 
 
 class LimitedBytesIO(BytesIO):
-    """Version of BytesIO that only accepts small reads/writes"""
+    """Version of BytesIO that only accepts small reads/writes."""
 
     def write(self, b):
         amt = min(len(b), 100)
@@ -40,29 +41,27 @@ def test_overwrite_with_memory_file(outdir):
     pdf.save(outdir / 'example.pdf')
 
 
-@pytest.mark.skipif(sys.version_info < (3, 6), reason='pathlib and shutil')
 def test_overwrite_input(resources, outdir):
     copy(resources / 'sandwich.pdf', outdir / 'sandwich.pdf')
-    p = Pdf.open(outdir / 'sandwich.pdf')
-    with pytest.raises(ValueError, match=r'overwrite input file'):
-        p.save(outdir / 'sandwich.pdf')
+    with Pdf.open(outdir / 'sandwich.pdf') as p:
+        with pytest.raises(ValueError, match=r'overwrite input file'):
+            p.save(outdir / 'sandwich.pdf')
 
 
-@pytest.mark.skipif(sys.version_info < (3, 6), reason='pathlib')
 def test_fail_only_overwrite_input_check(monkeypatch, resources, outdir):
     copy(resources / 'sandwich.pdf', outdir / 'sandwich.pdf')
-    p = Pdf.open(outdir / 'sandwich.pdf')
+    with Pdf.open(outdir / 'sandwich.pdf') as p:
 
-    def mockraise(*args):
-        raise OSError("samefile mocked")
+        def mockraise(*args):
+            raise OSError("samefile mocked")
 
-    monkeypatch.setattr(os.path, 'samefile', mockraise)
-    with pytest.raises(OSError, match=r'samefile mocked'):
-        p.save(outdir / 'wouldwork.pdf')
+        monkeypatch.setattr(os.path, 'samefile', mockraise)
+        with pytest.raises(OSError, match=r'samefile mocked'):
+            p.save(outdir / 'wouldwork.pdf')
 
 
 class BadBytesIO(BytesIO):
-    """Version of BytesIO that reports more bytes written than actual"""
+    """Version of BytesIO that reports more bytes written than actual."""
 
     def write(self, b):
         super().write(b)
@@ -70,7 +69,7 @@ class BadBytesIO(BytesIO):
 
 
 class WrongTypeBytesIO(BytesIO):
-    """Returns wrong type"""
+    """Returns wrong type."""
 
     def write(self, b):  # pylint: disable=unused-argument
         return None  # most likely wrong return type
@@ -99,8 +98,10 @@ def test_invalid_output_stream(sandwich, bio_class, exc_type):
 def file_descriptor_is_open_for():
     if sys.platform == 'win32' or sys.platform.startswith('freebsd'):
         pytest.skip(
-            msg="psutil documentation warns that .open_files() has problems on these"
+            "psutil documentation warns that .open_files() has problems on these"
         )
+    elif sys.implementation.name == 'pypy':
+        pytest.skip("randomly fails on pypy")
 
     def _file_descriptor_is_open_for(path):
         process = psutil.Process()
@@ -111,7 +112,7 @@ def file_descriptor_is_open_for():
 
 def test_open_named_file_closed(resources, file_descriptor_is_open_for):
     path = resources / 'pal.pdf'
-    pdf = Pdf.open(path)
+    pdf = Pdf.open(path)  # no with clause
     assert file_descriptor_is_open_for(path)
 
     pdf.close()
@@ -123,7 +124,7 @@ def test_open_named_file_closed(resources, file_descriptor_is_open_for):
 def test_streamed_file_not_closed(resources, file_descriptor_is_open_for):
     path = resources / 'pal.pdf'
     stream = path.open('rb')
-    pdf = Pdf.open(stream)
+    pdf = Pdf.open(stream)  # no with clause
     assert file_descriptor_is_open_for(path)
 
     pdf.close()
@@ -169,7 +170,6 @@ class ExpectedError(Exception):
     pass
 
 
-@pytest.mark.skipif(sys.version_info < (3, 6), reason='pathlib')
 def test_file_without_fileno(resources):
     class FileWithoutFileNo(FileIO):
         def fileno(self):
@@ -181,17 +181,17 @@ def test_file_without_fileno(resources):
 
     # Confirm we automatically fallback to stream
     with Pdf.open(f, access_mode=pikepdf._qpdf.AccessMode.mmap) as pdf:
-        pass
+        assert pdf.filename
 
 
 def test_file_deny_mmap(resources, monkeypatch):
     import mmap
 
-    def raises_ioerror(*args, **kwargs):
-        raise IOError("This file is temporarily not mmap-able")
+    def raises_oserror(*args, **kwargs):
+        raise OSError("This file is temporarily not mmap-able")
 
-    monkeypatch.setattr(mmap, 'mmap', raises_ioerror)
-    with pytest.raises(IOError):
+    monkeypatch.setattr(mmap, 'mmap', raises_oserror)
+    with pytest.raises(OSError):
         Pdf.open(resources / 'pal.pdf', access_mode=pikepdf._qpdf.AccessMode.mmap_only)
 
     with Pdf.open(
@@ -200,12 +200,61 @@ def test_file_deny_mmap(resources, monkeypatch):
         assert len(pdf.pages) == 1
 
 
-@pytest.mark.skipif(sys.version_info < (3, 6), reason='pathlib')
 def test_mmap_only_file(resources):
     class UnreadableFile(FileIO):
         def readinto(self, *args):
             raise ExpectedError("can't read, you have to mmap me")
 
+        read = readinto  # PyPy uses read() not readinto()
+
     f = UnreadableFile(resources / 'pal.pdf', 'rb')
     with pytest.raises(ExpectedError):
         Pdf.open(f, access_mode=pikepdf._qpdf.AccessMode.stream)
+
+
+def test_save_bytesio(resources, outpdf):
+    with Pdf.open(resources / 'fourpages.pdf') as input_:
+        pdf = Pdf.new()
+        for page in input_.pages:
+            pdf.pages.append(page)
+        bio = BytesIO()
+        pdf.save(bio, static_id=True)
+        bio_value = bio.getvalue()
+        assert bio_value != b''
+        pdf.save(outpdf, static_id=True)
+        assert outpdf.read_bytes() == bio_value
+
+
+@pytest.mark.skipif(
+    hasattr(os, 'geteuid') and os.geteuid() == 0, reason="root can override permissions"
+)
+def test_save_failure(sandwich, outdir):
+    dest = outdir / 'notwritable.pdf'
+
+    # This should work on Windows since Python maps the read-only bit
+    dest.touch(mode=0o444, exist_ok=False)
+    if dest.stat().st_mode & 0o400 != 0o400:
+        pytest.skip("Couldn't create a read-only file")
+
+    # Now try to overwrite
+    with pytest.raises(PermissionError, match="denied"):
+        sandwich.save(dest)
+
+
+def test_stop_iteration_on_close(resources):
+    class StopIterationOnClose(BytesIO):
+        def close(self):
+            raise StopIteration('To simulate weird generator behavior')
+
+    # Inspired by https://github.com/pikepdf/pikepdf/issues/114
+    stream = StopIterationOnClose((resources / 'pal-1bit-trivial.pdf').read_bytes())
+    pdf = Pdf.open(stream)  # no with clause
+    pdf.close()
+
+
+def test_read_after_close(resources):
+    pdf = Pdf.open(resources / 'pal.pdf')  # no with clause
+    contents = pdf.pages[0].Contents
+    pdf.close()
+    with pytest.raises(PdfError, match="closed input source"):
+        contents.read_raw_bytes()
