@@ -1,10 +1,11 @@
 import os
 import re
-import xml.etree.ElementTree as ET
 from datetime import datetime, timedelta, timezone
 from pathlib import Path
+from xml.etree import ElementTree as ET
 
 import pytest
+from conftest import skip_if_pypy
 from hypothesis import assume, example, given, settings
 from hypothesis import strategies as st
 from hypothesis.strategies import integers
@@ -20,48 +21,55 @@ from pikepdf.models.metadata import (
     decode_pdf_date,
 )
 
-try:
-    from libxmp import XMPMeta, XMPError
-except Exception:  # throws libxmp.ExempiLoadError pylint: disable=broad-except
-    XMPMeta, XMPError = None, None
-
-needs_libxmp = pytest.mark.skipif(
-    os.name == 'nt' or not XMPMeta, reason="test requires libxmp"
-)
-
 pytestmark = pytest.mark.filterwarnings('ignore:.*XMLParser.*:DeprecationWarning')
 
 # pylint: disable=redefined-outer-name,pointless-statement
 
 
+@pytest.fixture
+def libxmp_meta():
+    try:
+        libxmp = pytest.importorskip('libxmp')
+    except Exception as e:  # pylint: disable=broad-except
+        # libxmp raises a nonstandard exception sometimes so the standard
+        # pytest.importorskip doesn't trap it *irritated sigh*.
+        pytest.skip(str(e))
+    return libxmp.XMPMeta
+
+
 @pytest.fixture
 def vera(resources):
     # Has XMP but no docinfo
-    return Pdf.open(resources / 'veraPDF test suite 6-2-10-t02-pass-a.pdf')
+    with Pdf.open(resources / 'veraPDF test suite 6-2-10-t02-pass-a.pdf') as pdf:
+        yield pdf
 
 
 @pytest.fixture
 def graph(resources):
     # Has XMP and docinfo, all standard format XMP
-    return Pdf.open(resources / 'graph.pdf')
+    with Pdf.open(resources / 'graph.pdf') as pdf:
+        yield pdf
 
 
 @pytest.fixture
 def sandwich(resources):
     # Has XMP, docinfo, <?adobe-xap-filters esc="CRLF"?>, shorthand attribute XMP
-    return Pdf.open(resources / 'sandwich.pdf')
+    with Pdf.open(resources / 'sandwich.pdf') as pdf:
+        yield pdf
 
 
 @pytest.fixture
 def trivial(resources):
     # Has no XMP or docinfo
-    return Pdf.open(resources / 'pal-1bit-trivial.pdf')
+    with Pdf.open(resources / 'pal-1bit-trivial.pdf') as pdf:
+        yield pdf
 
 
 @pytest.fixture
 def invalid_creationdate(resources):
     # Has nuls in docinfo, old PDF
-    return Pdf.open(resources / 'invalid_creationdate.pdf')
+    with Pdf.open(resources / 'invalid_creationdate.pdf') as pdf:
+        yield pdf
 
 
 def test_lowlevel(sandwich):
@@ -100,8 +108,8 @@ def test_no_info(vera, outdir):
     assert vera.docinfo.is_indirect, "/Info must be an indirect object"
     vera.save(outdir / 'out.pdf')
 
-    new = Pdf.open(outdir / 'out.pdf')
-    assert new.docinfo['/Creator'] == creator
+    with Pdf.open(outdir / 'out.pdf') as new:
+        assert new.docinfo['/Creator'] == creator
 
 
 def test_update_info(graph, outdir):
@@ -109,23 +117,32 @@ def test_update_info(graph, outdir):
     graph.docinfo['/Title'] = new_title
     graph.save(outdir / 'out.pdf')
 
-    new = Pdf.open(outdir / 'out.pdf')
-    assert new.docinfo['/Title'] == new_title
-    assert graph.docinfo['/Author'] == new.docinfo['/Author']
+    with Pdf.open(outdir / 'out.pdf') as new:
+        assert new.docinfo['/Title'] == new_title
+        assert graph.docinfo['/Author'] == new.docinfo['/Author']
 
-    with pytest.raises(ValueError):
-        new.docinfo = Dictionary({'/Keywords': 'bob'})
+        with pytest.raises(ValueError):
+            new.docinfo = Dictionary({'/Keywords': 'bob'})
 
-    new.docinfo = graph.make_indirect(Dictionary({'/Keywords': 'bob'}))
-    assert new.docinfo.is_indirect, "/Info must be an indirect object"
+        new.docinfo = graph.make_indirect(Dictionary({'/Keywords': 'bob'}))
+        assert new.docinfo.is_indirect, "/Info must be an indirect object"
 
 
 def test_copy_info(vera, graph, outdir):
-    vera.docinfo = vera.copy_foreign(graph.docinfo)
+    vera.docinfo = vera.copy_foreign(graph.docinfo)  # vera has no docinfo
     assert vera.docinfo.is_indirect, "/Info must be an indirect object"
     vera.save(outdir / 'out.pdf')
 
 
+def test_del_info(graph, outpdf):
+    assert Name.Info in graph.trailer
+    del graph.docinfo
+    assert Name.Info not in graph.trailer
+    graph.save(outpdf)
+    with Pdf.open(outpdf) as new:
+        assert Name.Info not in new.trailer
+
+
 def test_add_new_xmp_and_mark(trivial):
     with trivial.open_metadata(
         set_pikepdf_as_editor=False, update_docinfo=False
@@ -153,6 +170,8 @@ def test_update_docinfo(vera):
     # Test delete propagation
     with vera.open_metadata(set_pikepdf_as_editor=False, update_docinfo=True) as xmp:
         del xmp['dc:creator']
+        with pytest.raises(KeyError, match='dc:nonexistent'):
+            del xmp['dc:nonexistent']
     assert 'dc:creator' not in xmp
     assert Name.Author not in vera.docinfo
 
@@ -162,14 +181,14 @@ def test_update_docinfo(vera):
 )
 def test_roundtrip(filename):
     try:
-        pdf = Pdf.open(filename)
+        with Pdf.open(filename) as pdf:
+            with pdf.open_metadata() as xmp:
+                for k in xmp.keys():
+                    if 'Date' not in k:
+                        xmp[k] = 'A'
+            assert '<?xpacket' not in str(xmp)
     except PasswordError:
         return
-    with pdf.open_metadata() as xmp:
-        for k in xmp.keys():
-            if not 'Date' in k:
-                xmp[k] = 'A'
-    assert '<?xpacket' not in str(xmp)
 
 
 def test_build_metadata(trivial, graph, outdir):
@@ -177,18 +196,17 @@ def test_build_metadata(trivial, graph, outdir):
         xmp.load_from_docinfo(graph.docinfo)
     trivial.save(outdir / 'tmp.pdf')
 
-    pdf = pikepdf.open(outdir / 'tmp.pdf')
-    assert pdf.Root.Metadata.Type == Name.Metadata
-    assert pdf.Root.Metadata.Subtype == Name.XML
-    with pdf.open_metadata(set_pikepdf_as_editor=False) as xmp:
-        assert 'pdf:Producer' not in xmp
-        xmp_date = xmp['xmp:CreateDate']
-        docinfo_date = decode_pdf_date(trivial.docinfo[Name.CreationDate])
-        assert xmp_date == docinfo_date.isoformat()
+    with pikepdf.open(outdir / 'tmp.pdf') as pdf:
+        assert pdf.Root.Metadata.Type == Name.Metadata
+        assert pdf.Root.Metadata.Subtype == Name.XML
+        with pdf.open_metadata(set_pikepdf_as_editor=False) as xmp:
+            assert 'pdf:Producer' not in xmp
+            xmp_date = xmp['xmp:CreateDate']
+            docinfo_date = decode_pdf_date(trivial.docinfo[Name.CreationDate])
+            assert xmp_date == docinfo_date.isoformat()
 
 
-@needs_libxmp
-def test_python_xmp_validate_add(trivial):
+def test_python_xmp_validate_add(trivial, libxmp_meta):
     with trivial.open_metadata() as xmp:
         xmp['dc:creator'] = ['Bob', 'Doug']
         xmp['dc:title'] = 'Title'
@@ -198,7 +216,7 @@ def test_python_xmp_validate_add(trivial):
     assert '<rdf:Seq><rdf:li>Bob</rdf:li><rdf:li>Doug</rdf:li>' in xmp_str
     assert '<rdf:Bag><rdf:li>Mackenzie</rdf:li>' in xmp_str
 
-    xmpmeta = XMPMeta(xmp_str=str(xmp))
+    xmpmeta = libxmp_meta(xmp_str=str(xmp))
     DC = XMP_NS_DC
     assert xmpmeta.does_array_item_exist(DC, 'creator', 'Bob')
     assert xmpmeta.does_array_item_exist(DC, 'creator', 'Doug')
@@ -206,28 +224,24 @@ def test_python_xmp_validate_add(trivial):
     assert xmpmeta.does_array_item_exist(DC, 'publisher', 'Mackenzie')
 
 
-@needs_libxmp
-def test_python_xmp_validate_change_list(graph):
+def test_python_xmp_validate_change_list(graph, libxmp_meta):
     with graph.open_metadata() as xmp:
         assert 'dc:creator' in xmp
         xmp['dc:creator'] = ['Dobby', 'Kreacher']
     assert str(xmp)
-    if not XMPMeta:
-        pytest.skip(msg='needs libxmp')
-    xmpmeta = XMPMeta(xmp_str=str(xmp))
+    xmpmeta = libxmp_meta(xmp_str=str(xmp))
     DC = XMP_NS_DC
     assert xmpmeta.does_array_item_exist(DC, 'creator', 'Dobby')
     assert xmpmeta.does_array_item_exist(DC, 'creator', 'Kreacher')
 
 
-@needs_libxmp
-def test_python_xmp_validate_change(sandwich):
+def test_python_xmp_validate_change(sandwich, libxmp_meta):
     with sandwich.open_metadata() as xmp:
         assert 'xmp:CreatorTool' in xmp
         xmp['xmp:CreatorTool'] = 'Creator'  # Exists as a xml tag text
         xmp['pdf:Producer'] = 'Producer'  # Exists as a tag node
     assert str(xmp)
-    xmpmeta = XMPMeta(xmp_str=str(xmp))
+    xmpmeta = libxmp_meta(xmp_str=str(xmp))
     assert xmpmeta.does_property_exist(XMP_NS_XMP, 'CreatorTool')
     assert xmpmeta.does_property_exist(XMP_NS_PDF, 'Producer')
 
@@ -249,9 +263,9 @@ def test_decode_pdf_date():
 
 def test_date_docinfo_from_xmp():
     VALS = [
-        ('2018-12-04T03:02:01', "20181204030201"),
-        ('2018-12-15T07:36:43Z', "20181215073643+00'00'"),
-        ('2018-12-04T03:02:01-01:00', "20181204030201-01'00'"),
+        ('2018-12-04T03:02:01', "D:20181204030201"),
+        ('2018-12-15T07:36:43Z', "D:20181215073643+00'00'"),
+        ('2018-12-04T03:02:01-01:00', "D:20181204030201-01'00'"),
     ]
     for xmp_val, docinfo_val in VALS:
         assert DateConverter.docinfo_from_xmp(xmp_val) == docinfo_val
@@ -269,7 +283,7 @@ def test_date_docinfo_from_xmp():
 def test_random_dates(year, month, day, hour, mins, sec):
     date_args = year, month, day, hour, mins, sec
     xmp = '{:04d}-{:02d}-{:02d}T{:02d}:{:02d}:{:02d}'.format(*date_args)
-    docinfo = '{:04d}{:02d}{:02d}{:02d}{:02d}{:02d}'.format(*date_args)
+    docinfo = 'D:{:04d}{:02d}{:02d}{:02d}{:02d}{:02d}'.format(*date_args)
 
     try:
         converted = DateConverter.docinfo_from_xmp(xmp)
@@ -300,7 +314,7 @@ def test_xpacket_generation(sandwich):
     assert xmpstr1.startswith(xpacket_begin)
 
     with sandwich.open_metadata() as xmp:
-        xmp['dc:creator'] = 'Foo'
+        xmp['dc:creator'] = ['Foo']
 
     xmpstr2 = sandwich.Root.Metadata.read_bytes()
     assert xmpstr2.startswith(xpacket_begin)
@@ -365,6 +379,16 @@ def test_docinfo_problems(sandwich, invalid_creationdate):
         assert 'could not be updated' in warned[0].message.args[0]
 
 
+def test_docinfo_delete_missing(sandwich):
+    with sandwich.open_metadata() as m:
+        d = Dictionary(Creator="test creator")
+        assert 'xmp:CreateDate' in m
+        assert m['xmp:CreatorTool'] != 'test creator'
+        m.load_from_docinfo(d, delete_missing=True)
+        assert m['xmp:CreatorTool'] == 'test creator'
+        assert 'xmp:CreateDate' not in m
+
+
 def test_present_bug_empty_tags(trivial):
     trivial.Root.Metadata = Stream(
         trivial,
@@ -379,7 +403,7 @@ def test_present_bug_empty_tags(trivial):
         """,
     )
     with trivial.open_metadata(update_docinfo=True) as meta:
-        pass
+        assert len(meta) > 0
     assert Name.Author not in trivial.docinfo
 
 
@@ -455,6 +479,7 @@ def test_degenerate_xml_recoverable(trivial, xml):
 
 @settings(deadline=None)
 @given(st.integers(min_value=1, max_value=1350))
+@example(531)
 @example(548)
 @example(1154)
 @example(1155)
@@ -462,30 +487,30 @@ def test_degenerate_xml_recoverable(trivial, xml):
 @example(1303)
 @pytest.mark.filterwarnings('ignore:The DocumentInfo field')
 def test_truncated_xml(resources, idx):
-    sandwich = Pdf.open(resources / 'sandwich.pdf')
-    data = sandwich.Root.Metadata.read_bytes()
-    assume(idx < len(data))
+    with Pdf.open(resources / 'sandwich.pdf') as sandwich:
+        data = sandwich.Root.Metadata.read_bytes()
+        assume(idx < len(data))
 
-    sandwich.Root.Metadata = sandwich.make_stream(data[0:idx])
-    try:
-        with sandwich.open_metadata(strict=True) as xmp:
-            xmp['pdfaid:part'] = '5'
-    except (XMLSyntaxError, AssertionError):
-        pass
-
-    with sandwich.open_metadata(strict=False) as xmp:
-        xmp['pdfaid:part'] = '7'
-
-
-@needs_libxmp
-def test_pdf_version_update(graph, outdir):
-    def get_xmp_version(filename):
-        meta = pikepdf.open(filename).open_metadata()
-        xmp = XMPMeta(xmp_str=str(meta))
+        sandwich.Root.Metadata = sandwich.make_stream(data[0:idx])
         try:
-            return xmp.get_property('http://ns.adobe.com/pdf/1.3/', 'PDFVersion')
-        except XMPError:
-            return ''
+            with sandwich.open_metadata(strict=True) as xmp:
+                xmp['pdfaid:part'] = '5'
+        except (XMLSyntaxError, AssertionError):
+            pass
+
+        with sandwich.open_metadata(strict=False) as xmp:
+            xmp['pdfaid:part'] = '7'
+
+
+def test_pdf_version_update(graph, outdir, libxmp_meta):
+    def get_xmp_version(filename):
+        with pikepdf.open(filename) as pdf:
+            meta = pdf.open_metadata()
+            xmp = libxmp_meta(xmp_str=str(meta))
+            try:
+                return xmp.get_property('http://ns.adobe.com/pdf/1.3/', 'PDFVersion')
+            except Exception:
+                return ''
 
     # We don't update PDFVersion unless it is present, even if we change the PDF version
     graph.save(
@@ -514,17 +539,18 @@ def test_pdf_version_update(graph, outdir):
 
 def test_extension_level(trivial, outpdf):
     trivial.save(outpdf, min_version=('1.6', 314159))
-    pdf = pikepdf.open(outpdf)
-    assert pdf.pdf_version >= '1.6' and pdf.extension_level == 314159
+    with pikepdf.open(outpdf) as pdf:
+        assert pdf.pdf_version >= '1.6' and pdf.extension_level == 314159
 
     trivial.save(outpdf, force_version=('1.7', 42))
-    pdf = pikepdf.open(outpdf)
-    assert pdf.pdf_version == '1.7' and pdf.extension_level == 42
+    with pikepdf.open(outpdf) as pdf:
+        assert pdf.pdf_version == '1.7' and pdf.extension_level == 42
 
     with pytest.raises(TypeError):
         trivial.save(outpdf, force_version=('1.7', 'invalid extension level'))
 
 
+@settings(deadline=500.0)
 @given(
     st.dictionaries(
         keys=st.sampled_from(
@@ -540,9 +566,14 @@ def test_extension_level(trivial, outpdf):
                 "/Dummy",
             ]
         ),
-        values=st.binary(),
+        values=st.recursive(
+            st.none() | st.binary(max_size=16) | st.booleans(),
+            lambda children: st.lists(children, min_size=0, max_size=4),
+            max_leaves=2,
+        ),
     )
 )
+@skip_if_pypy
 def test_random_docinfo(docinfo):
     p = pikepdf.new()
     with p.open_metadata() as m:
@@ -556,6 +587,51 @@ def test_random_docinfo(docinfo):
             ET.fromstring(str(m))  # ensure we can parse it
 
 
+@given(
+    st.dictionaries(
+        keys=st.sampled_from(
+            [
+                "/Author",
+                "/Subject",
+                "/Title",
+                "/Keywords",
+                "/Producer",
+                "/Creator",
+            ]
+        ),
+        values=st.none() | st.characters(blacklist_categories=('Cs',)),
+    )
+)
+@skip_if_pypy
+def test_random_valid_docinfo(docinfo):
+    p = pikepdf.new()
+    with p.open_metadata() as m:
+        pdf_docinfo = pikepdf.Dictionary(docinfo)
+
+        m.load_from_docinfo(pdf_docinfo, raise_failure=True)
+        ET.fromstring(str(m))  # ensure we can parse it
+
+
+@pytest.mark.parametrize('author', ['', 'King, S.'])
+def test_issue_162(trivial, author):
+    trivial.Root.Metadata = Stream(
+        trivial,
+        b"""
+        <?xpacket begin="\xef\xbb\xbf" id="W5M0MpCehiHzreSzNTczkc9d"?>
+        <rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
+                xmlns:xmp="http://ns.adobe.com/xap/1.0/"
+                xmlns:dc="http://purl.org/dc/elements/1.1/">
+        <rdf:Description rdf:about="" xmlns:dc="http://purl.org/dc/elements/1.1/" dc:creator="Foo"></rdf:Description>
+        </rdf:RDF>
+        <?xpacket end="w"?>""",
+    )
+    with trivial.open_metadata() as m:
+        docinfo = pikepdf.Dictionary(Author=author)
+        with pytest.warns(UserWarning, match=r'Merging elements'):
+            m.load_from_docinfo(docinfo, raise_failure=True)
+        assert m['dc:creator'] == [author]
+
+
 def test_set_empty_string(graph):
     with graph.open_metadata() as m:
         m['dc:title'] = 'a'
@@ -587,3 +663,68 @@ def test_issue_100(trivial):
         UserWarning, match="no XMP equivalent"
     ):
         m.load_from_docinfo({'/AAPL:Example': pikepdf.Array([42])})
+
+
+def test_issue_135_title_rdf_bag(trivial):
+    with trivial.open_metadata(update_docinfo=True) as xmp, pytest.warns(
+        UserWarning, match="Merging elements"
+    ):
+        xmp['dc:title'] = {'Title 1', 'Title 2'}
+    with trivial.open_metadata(update_docinfo=False) as xmp:
+        assert b'Title 1; Title 2</rdf:li></rdf:Alt></dc:title>' in xmp._get_xml_bytes()
+
+
+def test_xmp_metadatadate_timezone(sandwich, outpdf):
+    with sandwich.open_metadata():
+        pass
+    sandwich.save(outpdf)
+
+    machine_tz = datetime.utcnow().astimezone().tzinfo
+
+    with Pdf.open(outpdf) as pdf:
+        with pdf.open_metadata() as m:
+            dt = datetime.fromisoformat(m['xmp:MetadataDate'])
+            assert dt.tzinfo is not None
+            assert dt.tzinfo == machine_tz
+
+
+def test_modify_not_opened(graph):
+    m = graph.open_metadata()
+    with pytest.raises(RuntimeError, match='not opened for editing'):
+        m['pdf:Producer'] = 'pytest'
+
+
+def test_exception_undoes_edits(graph):
+    try:
+        with graph.open_metadata() as m:
+            m['dc:format'] = 'application/pdf-demo'
+            raise ValueError('test_exception_aborts_edits')
+    except ValueError as e:
+        if 'test_exception_aborts_edits' not in str(e):
+            raise
+        m = graph.open_metadata()
+        assert m['dc:format'] != 'application/pdf-demo'
+
+
+def test_xxe(trivial, outdir):
+    secret = outdir / 'secret.txt'
+    secret.write_text("This is a secret")
+    trivial.Root.Metadata = Stream(
+        trivial,
+        b"""\
+<?xpacket begin='\xef\xbb\xbf' id='W5M0MpCehiHzreSzNTczkc9d'?>
+<!DOCTYPE rdf:RDF [<!ENTITY xxe SYSTEM "file://%s">]>
+<x:xmpmeta xmlns:x='adobe:ns:meta/' x:xmptk='Image'>
+<rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#">
+<note>
+<to>&xxe;</to>
+<from>xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx</from>
+</note>
+</rdf:RDF>
+</x:xmpmeta>
+<?xpacket end='w'?>
+    """
+        % os.fsencode(secret),
+    )
+    with trivial.open_metadata() as m:
+        assert 'This is a secret' not in str(m)
