@@ -1,35 +1,57 @@
 import gc
 from contextlib import suppress
 from shutil import copy
-from sys import getrefcount as refcount
+from typing import Type, ValuesView
+
+try:
+    from sys import getrefcount as refcount
+except ImportError:
+
+    def refcount(_x):  # type: ignore
+        return 1
+
 
 import pytest
+from conftest import skip_if_pypy
 
-from pikepdf import Array, Dictionary, Name, Pdf, PdfMatrix, Stream
+from pikepdf import (
+    Array,
+    Dictionary,
+    Name,
+    Page,
+    Pdf,
+    PdfMatrix,
+    Stream,
+    __libqpdf_version__,
+)
+from pikepdf._cpphelpers import label_from_label_dict
 
 # pylint: disable=redefined-outer-name,pointless-statement
 
 
 @pytest.fixture
 def graph(resources):
-    return Pdf.open(resources / 'graph.pdf')
+    with Pdf.open(resources / 'graph.pdf') as pdf:
+        yield pdf
 
 
 @pytest.fixture
 def fourpages(resources):
-    return Pdf.open(resources / 'fourpages.pdf')
+    with Pdf.open(resources / 'fourpages.pdf') as pdf:
+        yield pdf
 
 
 @pytest.fixture
 def sandwich(resources):
-    return Pdf.open(resources / 'sandwich.pdf')
+    with Pdf.open(resources / 'sandwich.pdf') as pdf:
+        yield pdf
 
 
 def test_split_pdf(fourpages, outdir):
     for n, page in enumerate(fourpages.pages):
         outpdf = Pdf.new()
         outpdf.pages.append(page)
-        outpdf.save(outdir / "page{}.pdf".format(n + 1))
+        outpdf.save(outdir / f"page{n + 1}.pdf")
 
     assert len([f for f in outdir.iterdir() if f.name.startswith('page')]) == 4
 
@@ -82,26 +104,26 @@ def test_hard_replace_page(fourpages, graph, sandwich, outdir):
 
 
 def test_reverse_pages(resources, outdir):
-    q = Pdf.open(resources / "fourpages.pdf")
-    qr = Pdf.open(resources / "fourpages.pdf")
+    with Pdf.open(resources / "fourpages.pdf") as q, Pdf.open(
+        resources / "fourpages.pdf"
+    ) as qr:
+        lengths = [int(page.Contents.stream_dict.Length) for page in q.pages]
 
-    lengths = [int(page.Contents.stream_dict.Length) for page in q.pages]
+        qr.pages.reverse()
+        qr.save(outdir / "reversed.pdf")
 
-    qr.pages.reverse()
-    qr.save(outdir / "reversed.pdf")
+        for n, length in enumerate(lengths):
+            assert q.pages[n].Contents.stream_dict.Length == length
 
-    for n, length in enumerate(lengths):
-        assert q.pages[n].Contents.stream_dict.Length == length
-
-    for n, length in enumerate(reversed(lengths)):
-        assert qr.pages[n].Contents.stream_dict.Length == length
+        for n, length in enumerate(reversed(lengths)):
+            assert qr.pages[n].Contents.stream_dict.Length == length
 
 
+@skip_if_pypy
 def test_evil_page_deletion(resources, outdir):
-    # str needed for py<3.6
-    copy(str(resources / 'sandwich.pdf'), str(outdir / 'sandwich.pdf'))
+    copy(resources / 'sandwich.pdf', outdir / 'sandwich.pdf')
 
-    src = Pdf.open(outdir / 'sandwich.pdf')
+    src = Pdf.open(outdir / 'sandwich.pdf')  # no with clause
     pdf = Pdf.open(resources / 'graph.pdf')
 
     assert refcount(src) == 2
@@ -148,6 +170,12 @@ def test_extend_delete(sandwich, fourpages, outdir):
     pdf.save(outdir / 'out.pdf')
 
 
+def test_extend_with_nonpage(sandwich):
+    pdf = sandwich
+    with pytest.raises(TypeError, match="only pikepdf"):
+        pdf.pages.extend([42])
+
+
 def test_slice_unequal_replacement(fourpages, sandwich, outdir):
     pdf = fourpages
     pdf2 = sandwich
@@ -208,19 +236,6 @@ def test_one_based_pages(fourpages):
         pdf.pages.p(-1)
 
 
-def test_page_contents_add(graph, outdir):
-    pdf = graph
-
-    mat = PdfMatrix().rotated(45)
-
-    stream1 = Stream(pdf, b'q ' + mat.encode() + b' cm')
-    stream2 = Stream(pdf, b'Q')
-
-    pdf.pages[0].page_contents_add(stream1, True)
-    pdf.pages[0].page_contents_add(stream2, False)
-    pdf.save(outdir / 'out.pdf')
-
-
 def test_bad_access(fourpages):
     pdf = fourpages
     with pytest.raises(IndexError):
@@ -256,20 +271,32 @@ def test_concatenate(resources, outdir):
         output_pdf = Pdf.new()
         for i in range(n):
             print(i)
-            pdf_page = Pdf.open(resources / 'pal.pdf')
-            output_pdf.pages.extend(pdf_page.pages)
-        output_pdf.save(outdir / '{}.pdf'.format(n))
+            with Pdf.open(resources / 'pal.pdf') as pdf_page:
+                output_pdf.pages.extend(pdf_page.pages)
+        output_pdf.save(outdir / f'{n}.pdf')
 
     concatenate(5)
 
 
 def test_emplace(fourpages):
     p0_objgen = fourpages.pages[0].objgen
+    fourpages.pages[0].SpecialKey = "This string will be deleted"
+    fourpages.pages[0].Parent = Name.ParentWillBeRetained
+    repr_fourpages_1 = repr(fourpages.pages[1])
+
     fourpages.pages[0].emplace(fourpages.pages[1])
-    assert p0_objgen == fourpages.pages[0].objgen
-    assert fourpages.pages[0].keys() == fourpages.pages[1].keys()
+
+    assert p0_objgen == fourpages.pages[0].objgen, "objgen modified"
+    assert fourpages.pages[0].keys() == fourpages.pages[1].keys(), "Keys mismatched"
     for k in fourpages.pages[0].keys():
-        assert fourpages.pages[0][k] == fourpages.pages[1][k]
+        if k != Name.Parent:
+            assert fourpages.pages[0][k] == fourpages.pages[1][k], "Key not copied"
+        else:
+            assert (
+                fourpages.pages[0][k] == Name.ParentWillBeRetained
+            ), "Retained key not retained"
+    assert Name.SpecialKey not in fourpages.pages[0]
+    assert repr_fourpages_1 == repr(fourpages.pages[1]), "Source page was modified"
 
 
 def test_emplace_foreign(fourpages, sandwich):
@@ -308,11 +335,11 @@ def test_repeat(graph, outpdf):
     graph.save(outpdf)
 
 
-def test_add_foreign_twice(graph, outpdf):
+def test_add_twice_without_copy_foreign(graph, outpdf):
     out = Pdf.new()
-    out.pages.append(out.copy_foreign(graph.pages[0]))
+    out.pages.append(graph.pages[0])
     assert len(out.pages) == 1
-    out.pages.append(out.copy_foreign(graph.pages[0]))
+    out.pages.append(graph.pages[0])
     assert len(out.pages) == 2
     out.save(outpdf)
 
@@ -323,16 +350,16 @@ def test_repr_pagelist(fourpages):
 
 def test_foreign_copied_pages_are_true_copies(graph, outpdf):
     out = Pdf.new()
-    for n in range(4):
-        out.pages.append(out.copy_foreign(graph.pages[0]))
+    for _ in range(4):
+        out.pages.append(graph.pages[0])
 
     for n in [0, 2]:
         out.pages[n].Rotate = 180
 
     out.save(outpdf)
-    reopened = Pdf.open(outpdf)
-    assert reopened.pages[0].Rotate == 180
-    assert reopened.pages[1].get(Name.Rotate, 0) == 0
+    with Pdf.open(outpdf) as reopened:
+        assert reopened.pages[0].Rotate == 180
+        assert reopened.pages[1].get(Name.Rotate, 0) == 0
 
 
 def test_remove_onebased(fourpages):
@@ -357,23 +384,152 @@ def test_pages_wrong_type(fourpages):
 
 
 def test_page_splitting_generator(resources, tmp_path):
+    # https://github.com/pikepdf/pikepdf/issues/114
     def pdfs():
-        pdf = Pdf.open(resources / "content-stream-errors.pdf")
-        output = Pdf.new()
-        part = 1
-        for _idx, page in enumerate(pdf.pages):
-            if len(output.pages) == 2:
-                part_file = tmp_path / "part-{0}.pdf".format(part)
+        with Pdf.open(
+            resources / "content-stream-errors.pdf"
+        ) as pdf, Pdf.new() as output:
+            part = 1
+            for _idx, page in enumerate(pdf.pages):
+                if len(output.pages) == 2:
+                    part_file = tmp_path / f"part-{part}.pdf"
+                    output.save(part_file)
+                    yield part_file
+                    output = Pdf.new()
+                    part += 1
+                output.pages.append(page)
+            if len(output.pages) > 0:
+                part_file = tmp_path / f"part-{part}.pdf"
                 output.save(part_file)
                 yield part_file
-                output = Pdf.new()
-                part += 1
-            output.pages.append(page)
-        if len(output.pages) > 0:
-            part_file = tmp_path / "part-{0}.pdf".format(part)
-            output.save(part_file)
-            yield part_file
-        output.close()
 
     for pdf in pdfs():
         print(pdf)
+
+
+def test_page_index(fourpages):
+    for n, page in enumerate(fourpages.pages):
+        assert page.index == n
+        assert fourpages.pages.index(page) == n
+        assert fourpages.pages.index(page.obj) == n
+    del fourpages.pages[1]
+    for n, page in enumerate(fourpages.pages):
+        assert page.index == n
+        assert fourpages.pages.index(page.obj) == n
+
+
+def test_page_index_foreign_page(fourpages, sandwich):
+    with pytest.raises(ValueError, match="Page is not in this Pdf"):
+        fourpages.pages.index(sandwich.pages[0])
+
+    p3 = fourpages.pages[2]
+    assert p3.index == 2
+    fourpages.pages.insert(2, sandwich.pages[0])
+    assert fourpages.pages[2].index == 2
+    assert p3.index == 3
+
+    assert fourpages.pages.index(p3) == 3
+    assert fourpages.pages.index(Page(p3)) == 3  # Keep
+
+    with pytest.raises(ValueError, match="Page is not in this Pdf"):
+        # sandwich.pages[0] is still not "in" fourpages; it gets copied into it
+        assert fourpages.pages.index(sandwich.pages[0])
+
+
+@pytest.mark.parametrize(
+    'd, result',
+    [
+        (Dictionary(), ''),
+        (Dictionary(St=1), ''),
+        (Dictionary(S=Name.D, St=1), '1'),
+        (Dictionary(P='foo'), 'foo'),
+        (Dictionary(P='A', St=2), 'A'),
+        (Dictionary(P='A-', S=Name.D, St=2), 'A-2'),
+        (Dictionary(S=Name.R, St=42), 'XLII'),
+        (Dictionary(S=Name.r, St=1729), 'mdccxxix'),
+        (Dictionary(P="Appendix-", S=Name.a, St=261), 'Appendix-ja'),
+        (42, '42'),
+        (Dictionary(S=Name.R, St=-42), ValueError),
+        (Dictionary(S=Name.A, St=-42), ValueError),
+    ],
+)
+def test_page_label_dicts(d, result):
+    if isinstance(result, type) and issubclass(result, Exception):
+        with pytest.raises(result):
+            label_from_label_dict(d)
+    else:
+        assert label_from_label_dict(d) == result
+
+
+def test_externalize(resources):
+    with Pdf.open(resources / 'image-mono-inline.pdf') as p:
+        page = p.pages[0]
+        page.contents_coalesce()
+        assert b'BI' in page.obj.Contents.read_bytes(), "no inline image"
+
+        assert Name.XObject not in page.obj.Resources, "expected no xobjs"
+        page.externalize_inline_images()
+
+        assert Name.XObject in page.obj.Resources, "image not created"
+
+        pdfimagexobj = next(iter(p.pages[0].images.values()))
+        assert pdfimagexobj.Subtype == Name.Image
+
+        assert page.label == '1'
+
+
+def test_page_labels():
+    p = Pdf.new()
+    d = Dictionary(Type=Name.Page, MediaBox=[0, 0, 612, 792], Resources=Dictionary())
+    for n in range(5):
+        p.pages.append(d)
+        p.pages[n].Contents = Stream(p, b"BT (Page %s) Tj ET" % str(n).encode())
+
+    p.Root.PageLabels = p.make_indirect(
+        Dictionary(
+            Nums=Array(
+                [
+                    0,  # new label rules begin at index 0
+                    Dictionary(S=Name.r),  # use lowercase roman numerals, until...
+                    2,  # new label rules begin at index 2
+                    Dictionary(
+                        S=Name.D, St=42, P='Prefix-'
+                    ),  # label pages as 'Prefix-42', 'Prefix-43', ...
+                ]
+            )
+        )
+    )
+
+    labels = ['i', 'ii', 'Prefix-42', 'Prefix-43', 'Prefix-44']
+    for n in range(5):
+        page = p.pages[n]
+        assert page.label == labels[n]
+
+
+def test_unattached_page():
+    rawpage = Dictionary(
+        Type=Name.Page, MediaBox=[0, 0, 612, 792], Resources=Dictionary()
+    )
+    page = Page(rawpage)
+
+    with pytest.raises(ValueError, match='not attached'):
+        page.index
+    with pytest.raises(ValueError, match='not attached'):
+        page.label
+
+
+def test_unindexed_page(graph):
+    page = graph.pages[0]
+    del graph.pages[0]
+    with pytest.raises(ValueError, match='not consistently registered'):
+        page.index
+
+
+def test_page_from_objgen(graph):
+    assert graph.pages.from_objgen(graph.pages[0].objgen) == graph.pages[0]
+    assert (
+        graph.pages.from_objgen(graph.pages[0].objgen[0], graph.pages[0].objgen[1])
+        == graph.pages[0]
+    )
+    with pytest.raises(ValueError):
+        graph.pages.from_objgen(graph.pages[0].Contents.objgen)
