@@ -1,14 +1,12 @@
-import pytest
-from pikepdf import Pdf, PdfError
-
-import os
-import platform
-import shutil
-from contextlib import suppress
-from shutil import copy
 import gzip
 from pathlib import Path
 
+import pytest
+
+from pikepdf import Pdf, PdfError
+
+# pylint: disable=redefined-outer-name
+
 
 # Files with unknown copyright status can't be shared publicly
 PRIVATE_RESOURCES = Path(__file__).parent / 'resources' / 'private'
@@ -20,8 +18,8 @@ def private():
 
 
 pytestmark = pytest.mark.skipif(
-    not PRIVATE_RESOURCES.is_dir(),
-    reason='private resources not available')
+    not PRIVATE_RESOURCES.is_dir(), reason='private resources not available'
+)
 
 
 def test_pypdf2_issue_361(private):
