@@ -1,10 +1,20 @@
 import gc
-from sys import getrefcount as refcount
+
+try:
+    from sys import getrefcount as refcount
+except ImportError:
+
+    def refcount(_x):  # type: ignore
+        return 1
+
 
 import pytest
+from conftest import skip_if_pypy
 
 from pikepdf import Pdf
 
+pytestmark = skip_if_pypy
+
 # Try to do some things without blowing up
 
 
